from django.test import TestCase, override_settings
from maas.exceptions import *
from maas.models import *


class GenericTest(TestCase):

    def setUp(self):

        def buildMail():
            return Mail(sender='test@test.com', recipient='test@test.com', subject='subject', body='body')

        self.buildMail = buildMail

    def test_no_settings(self):
        with self.assertRaises(MaasConfigurationException):
            mail = self.buildMail()
            mail.send()

    @override_settings(MAAS={})
    def test_no_provider(self):
        with self.assertRaises(MaasConfigurationException):
            mail = self.buildMail()
            mail.send()

    @override_settings(MAAS={'PROVIDER': 'UNKNOWN'})
    def test_unknown_provider(self):
        with self.assertRaises(MaasConfigurationException):
            mail = self.buildMail()
            mail.send()


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-06 19:28
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('maas', '0002_auto_20170606_1857'),
    ]

    operations = [
        migrations.AddField(
            model_name='mail',
            name='delivered',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='mail',
            name='sender_title',
            field=models.CharField(default='', max_length=200),
        ),
    ]



from django.db import models
from django.conf import settings
from .exceptions import *
import requests

MAILGUN_V3_BASE = 'https://api.mailgun.net/v3'

MAIL_STATUSES = [
    (0, '')
]


class Mail(models.Model):
    sender = models.EmailField()
    sender_title = models.CharField(blank=True, null=True, default='', max_length=200)
    recipient = models.EmailField()
    subject = models.CharField(max_length=200)
    body = models.TextField(max_length=5000)
    delivered = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        verbose_name_plural = 'Mail'

    def _send_mailgun_v3(self):
        if 'DOMAIN' not in settings.MAAS:
            raise MaasConfigurationException('MAAS.DOMAIN must be defined in settings')

        if 'API_KEY' not in settings.MAAS:
            raise MaasConfigurationException('MAAS.DOMAIN must be defined in settings')

        url = '{}/{}/messages'.format(MAILGUN_V3_BASE, settings.MAAS['DOMAIN'])
        auth = ('api', settings.MAAS['API_KEY'])
        data = {
            'from': '{} <{}>'.format(self.sender_title, self.sender),
            'to': [ self.recipient ],
            'subject': self.subject,
            'text': self.body
        }

        res = requests.post(url, auth=auth, data=data)

        if res.status_code == 400:
            raise MaasUnknownException('Mailgun reported a bad request was made. Please report this error to django-maas.')

        if res.status_code == 401:
            raise MaasConfigurationException('MAAS.API_KEY seems to be invalid.')

        if res.status_code == 402:
            raise MaasUnknownException('Mailgun reported that the parameters were valid but the request failed. It is unclear what this means. Please report this error to django-maas.')

        if res.status_code == 404:
            raise MaasConfigurationException('MAAS.DOMAIN seems to be invalid,')

        if res.status_code in [500, 502, 503, 504]:
            raise MaasProviderException('Mailgun encountered an error on their end')

        try:
            res.raise_for_status()
        except:
            raise MaasUnknownException('Mailgun returned an undocumented status code {}. Please report this error to django-maas.'.format(res.status_code))

        self.delivered = True
        self.save()


    def send(self):
        provider_methods = {
            'MAILGUN_V3': self._send_mailgun_v3
        }

        if not hasattr(settings, 'MAAS'):
            raise MaasConfigurationException('MAAS must be defined in settings')

        if 'PROVIDER' not in settings.MAAS:
            raise MaasConfigurationException('MAAS.PROVIDER must be set')

        if settings.MAAS['PROVIDER'] not in provider_methods:
            raise MaasConfigurationException('MAAS.PROVIDER is invalid')

        provider_methods[settings.MAAS['PROVIDER']]()


from django.test import TestCase, override_settings
from maas.exceptions import *
from maas.models import *
import responses
import re


class MailgunV3Test(TestCase):

    def setUp(self):
        def buildMail():
            return Mail(sender='test@test.com', recipient='test@test.com', subject='subject', body='body')

        self.buildMail = buildMail

    @override_settings(MAAS={'PROVIDER': 'MAILGUN_V3'})
    def test_no_domain(self):
        with self.assertRaises(MaasConfigurationException):
            mail = self.buildMail()
            mail.send()

    @override_settings(MAAS={'PROVIDER': 'MAILGUN_V3', 'DOMAIN': 'test'})
    def test_no_api_key(self):
        with self.assertRaises(MaasConfigurationException):
            mail = self.buildMail()
            mail.send()

    @override_settings(MAAS={'PROVIDER': 'MAILGUN_V3', 'DOMAIN': 'test', 'API_KEY': 'test'})
    @responses.activate
    def test_invalid_api_key(self):
        responses.add(responses.POST, re.compile(r'^{}/test/messages'.format(MAILGUN_V3_BASE)), status=401)

        with self.assertRaises(MaasConfigurationException):
            mail = self.buildMail()
            mail.send()

    @override_settings(MAAS={'PROVIDER': 'MAILGUN_V3', 'DOMAIN': 'test', 'API_KEY': 'test'})
    @responses.activate
    def test_invalid_domain(self):
        responses.add(responses.POST, re.compile(r'^{}/test/messages'.format(MAILGUN_V3_BASE)), status=404)

        with self.assertRaises(MaasConfigurationException):
            mail = self.buildMail()
            mail.send()

    @override_settings(MAAS={'PROVIDER': 'MAILGUN_V3', 'DOMAIN': 'test', 'API_KEY': 'test'})
    @responses.activate
    def test_mailgun_error(self):
        for status in [500, 502, 503, 504]:
            responses.add(responses.POST, re.compile(r'^{}/test/messages'.format(MAILGUN_V3_BASE)), status=status)

            with self.assertRaises(MaasProviderException):
                mail = self.buildMail()
                mail.send()

    @override_settings(MAAS={'PROVIDER': 'MAILGUN_V3', 'DOMAIN': 'test', 'API_KEY': 'test'})
    @responses.activate
    def test_bad_request(self):
        responses.add(responses.POST, re.compile(r'^{}/test/messages'.format(MAILGUN_V3_BASE)), status=400)

        with self.assertRaises(MaasUnknownException):
            mail = self.buildMail()
            mail.send()

    @override_settings(MAAS={'PROVIDER': 'MAILGUN_V3', 'DOMAIN': 'test', 'API_KEY': 'test'})
    @responses.activate
    def test_bad_request(self):
        responses.add(responses.POST, re.compile(r'^{}/test/messages'.format(MAILGUN_V3_BASE)), status=407)

        with self.assertRaises(MaasUnknownException):
            mail = self.buildMail()
            mail.send()

    @override_settings(MAAS={'PROVIDER': 'MAILGUN_V3', 'DOMAIN': 'test', 'API_KEY': 'test'})
    @responses.activate
    def test_bad_request(self):
        responses.add(responses.POST, re.compile(r'^{}/test/messages'.format(MAILGUN_V3_BASE)), status=402)

        with self.assertRaises(MaasUnknownException):
            mail = self.buildMail()
            mail.send()

    @override_settings(MAAS={'PROVIDER': 'MAILGUN_V3', 'DOMAIN': 'test', 'API_KEY': 'test'})
    @responses.activate
    def test_successful_delivery(self):
        responses.add(responses.POST, re.compile(r'^{}/test/messages'.format(MAILGUN_V3_BASE)), status=200)
        mail = self.buildMail()
        mail.save()
        self.assertEqual(mail.delivered, False)
        mail.send()
        mail = Mail.objects.get(id=mail.id)
        self.assertEqual(mail.delivered, True)


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-06 18:57
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('maas', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='mail',
            name='body',
            field=models.TextField(default='', max_length=5000),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='mail',
            name='recipient',
            field=models.EmailField(default='test@test.com', max_length=254),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='mail',
            name='sender',
            field=models.EmailField(default='test@test.com', max_length=254),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='mail',
            name='subject',
            field=models.CharField(default='', max_length=200),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-06 20:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('maas', '0003_auto_20170606_1928'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='mail',
            options={'verbose_name_plural': 'Mail'},
        ),
        migrations.AddField(
            model_name='mail',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, default='1970-01-01 00:00:00'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='mail',
            name='updated_at',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name='mail',
            name='sender_title',
            field=models.CharField(blank=True, default='', max_length=200, null=True),
        ),
    ]



from django.contrib import admin
from .models import *


@admin.register(Mail)
class MailAdmin(admin.ModelAdmin):
    readonly_fields = [
        'sender',
        'sender_title',
        'recipient',
        'subject',
        'body',
        'delivered',
    ]

    list_display = [
        'recipient',
        'subject',
        'delivered',
        'created_at',
        'updated_at',
    ]

    search_fields = [
        'recipient'
    ]

    def has_add_permission(self, *args, **kwargs):
        return False

    def has_delete_permission(self, *args, **kwargs):
        return False


from setuptools import setup, find_packages


setup(
    name='maas',
    version='0.0.3',
    url='https://gitlab.com/matix-io/django-maas',
    license='BSD',
    description='Django wrapper for interfacing with e-Mail as a Service providers.',
    long_description='',
    author='Connor Bode',
    author_email='connor@matix.io',  # SEE NOTE BELOW (*)
    packages=find_packages(),
    install_requires=[],
    zip_safe=False,
    classifiers=[]
)